import graphene
from graphql_jwt.decorators import login_required, permission_required

from allianceauth.srp.models import SrpFleetMain

from .types import SrpFleetMainType


class Query:
    srp_get_fleets = graphene.List(SrpFleetMainType, all=graphene.Boolean(default_value=False))

    @login_required
    @permission_required('srp.access_srp')
    def resolve_srp_get_fleets(self, info, all):
        res = SrpFleetMain.objects.all()
        if not all:
            res = res.filter(fleet_srp_status="")
        return res



import graphene
from graphene_django import DjangoObjectType

from allianceauth.groupmanagement.models import GroupRequest, AuthGroup, RequestLog


class AuthGroupType(DjangoObjectType):
    group = graphene.Field('allianceauth_graphql.authentication.types.GroupType', required=True)

    class Meta:
        model = AuthGroup
        fields = ('group', 'description',)


class GroupRequestAddStatus(graphene.Enum):
    CANNOT_JOIN = 1
    ALREADY_MEMBER = 2
    JOINED = 3
    ALREADY_APPLIED = 4
    APPLIED = 5


class GroupRequestLeaveStatus(graphene.Enum):
    CANNOT_LEAVE = 1
    NOT_MEMBER = 2
    LEFT = 3
    PENDING_LEAVE_REQUEST = 4
    CREATED_LEAVE_REQUEST = 5


class GroupRequestType(DjangoObjectType):
    group = graphene.Field('allianceauth_graphql.authentication.types.GroupType', required=True)

    class Meta:
        model = GroupRequest


class GroupManagementType(graphene.ObjectType):
    leave_requests = graphene.List(GroupRequestType)
    accept_requests = graphene.List(GroupRequestType)
    auto_leave = graphene.Boolean()


class MemberType(graphene.ObjectType):
    user = graphene.Field('allianceauth_graphql.authentication.types.UserType')
    is_leader = graphene.Boolean()


class GroupMembershipListType(graphene.ObjectType):
    group = graphene.Field('allianceauth_graphql.authentication.types.GroupType')
    members = graphene.List(MemberType)


class GroupRequestLogType(graphene.Enum):
    REMOVED = 1
    LEAVE = 2
    JOIN = 3


class GroupRequestLogActionType(graphene.Enum):
    ACCEPT = 1
    REJECT = 2


class RequestLogType(DjangoObjectType):
    request_type = graphene.Field(GroupRequestLogType)
    requestor = graphene.Field('allianceauth_graphql.authentication.types.UserType')
    action = graphene.Field(GroupRequestLogActionType)
    group = graphene.Field('allianceauth_graphql.authentication.types.GroupType', required=True)

    class Meta:
        model = RequestLog
        fields = ('id', 'group', 'request_actor', 'date',)

    def resolve_request_type(self, info):
        if self.request_type is None:
            return GroupRequestLogType.REMOVED
        elif self.request_type is True:
            return GroupRequestLogType.LEAVE
        elif self.request_type is False:
            return GroupRequestLogType.JOIN

    def resolve_action(self, info):
        if self.request_type is not None:
            if self.action is True:
                return GroupRequestLogActionType.ACCEPT
            elif self.action is False:
                return GroupRequestLogActionType.REJECT


class GroupMembershipAuditType(graphene.ObjectType):
    group = graphene.Field('allianceauth_graphql.authentication.types.GroupType')
    entries = graphene.List(RequestLogType)



import graphene

from django.db.models import Q
from django.utils import timezone

from allianceauth.timerboard.models import Timer

from .types import StructureTimerType


class Query:
    tmr_future_timers = graphene.List(StructureTimerType, required=True)
    tmr_past_timers = graphene.List(StructureTimerType, required=True)

    def resolve_tmr_future_timers(self, info):
        return Timer.objects.filter(
            (
                Q(corp_timer=True) &
                Q(eve_corp=info.context.user.profile.main_character.corporation)
            ) |
            Q(corp_timer=False),
            eve_time__gte=timezone.now()
        )

    def resolve_tmr_past_timers(self, info):
        return Timer.objects.filter(
            (
                Q(corp_timer=True) &
                Q(eve_corp=info.context.user.profile.main_character.corporation)
            ) |
            Q(corp_timer=False),
            eve_time__lt=timezone.now()
        )



import graphene
from graphql_jwt.decorators import login_required, permission_required

from django.utils import timezone
from django.db.models import Q
from django.contrib.auth import get_user_model

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter

from allianceauth_pve.models import Rotation
from allianceauth_pve.actions import running_averages
from allianceauth_graphql.eveonline.types import EveCharacterType

from .types import RotationType, EntryType, RattingSummaryType


User = get_user_model()


class Query:
    rotation = graphene.Field(RotationType, id=graphene.Int(required=True))
    closed_rotations = graphene.List(RotationType)
    char_running_averages = graphene.Field(RattingSummaryType, start_date=graphene.Date(required=True), end_date=graphene.Date())
    active_rotations = graphene.List(RotationType)
    search_rotation_characters = graphene.List(EveCharacterType, name=graphene.String(), exclude_characters_ids=graphene.List(graphene.Int))

    @login_required
    @permission_required('allianceauth_pve.access_pve')
    def resolve_rotation(self, info, id):
        return Rotation.objects.get(pk=id)

    @login_required
    @permission_required('allianceauth_pve.access_pve')
    def resolve_closed_rotations(self, info):
        return Rotation.objects.filter(is_closed=True).order_by('-closed_at')

    @login_required
    def resolve_char_running_averages(self, info, start_date, end_date=timezone.now()):
        return running_averages(info.context.user, start_date, end_date)

    @login_required
    @permission_required('allianceauth_pve.access_pve')
    def resolve_active_rotations(self, info):
        return Rotation.objects.filter(is_closed=False).order_by('-priority')

    @login_required
    @permission_required('allianceauth_pve.manage_entries')
    def resolve_search_rotation_characters(self, info, name=None, exclude_characters_ids=[]):
        ratting_users = User.objects.filter(
            Q(groups__permissions__codename='access_pve') |
            Q(user_permissions__codename='access_pve') |
            Q(profile__state__permissions__codename='access_pve'),
            profile__main_character__isnull=False,
        )

        ownerships = CharacterOwnership.objects.filter(user__in=ratting_users)
        results = EveCharacter.objects.filter(pk__in=ownerships.values('character'))

        if name:
            results = results.filter(character_name__icontains=name)

        if len(exclude_characters_ids) > 0:
            results = results.exclude(pk__in=exclude_characters_ids)

        return results



import graphene
from graphql_jwt.decorators import login_required, permission_required
from graphene_django.forms.mutation import DjangoFormMutation

from django.utils import timezone
from django.contrib.humanize.templatetags.humanize import intcomma

from allianceauth.srp.form import SrpFleetMainForm
from allianceauth.srp.models import SrpFleetMain, SrpUserRequest
from allianceauth.srp.views import random_string
from allianceauth.srp.managers import SRPManager
from allianceauth.srp.providers import esi
from allianceauth.notifications import notify

from ..decorators import permissions_required
from .types import SrpFleetMainType
from .forms import GQLSrpFleetUserRequestForm


class AddFleetMutation(DjangoFormMutation):
    class Meta:
        form_class = SrpFleetMainForm

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permissions_required(('auth.srp_management', 'srp.add_srpfleetmain'))
    def perferm_mutate(cls, form: SrpFleetMainForm, info):
        srp_fleet_main = SrpFleetMain()
        srp_fleet_main.fleet_name = form.cleaned_data['fleet_name']
        srp_fleet_main.fleet_doctrine = form.cleaned_data['fleet_doctrine']
        srp_fleet_main.fleet_time = form.cleaned_data['fleet_time']
        srp_fleet_main.fleet_srp_code = random_string(8)
        srp_fleet_main.fleet_commander = info.context.user.profile.main_character

        srp_fleet_main.save()

        return cls(ok=True)


class RemoveFleetMutation(graphene.Mutation):
    class Meta:
        fleet_id = graphene.ID(required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, fleet_id):
        SrpFleetMain.objects.filter(pk=fleet_id).delete()
        return cls(ok=True)


class DisableFleetMutation(graphene.Mutation):
    class Meta:
        fleet_id = graphene.ID(required=True)

    ok = graphene.Boolean()
    srp_fleet = graphene.Field(SrpFleetMainType)

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, fleet_id):
        srpfleetmain = SrpFleetMain.objects.get(pk=fleet_id)
        srpfleetmain.fleet_srp_code = ""
        srpfleetmain.save()
        return cls(ok=True, srp_fleet=srpfleetmain)


class EnableFleetMutation(graphene.Mutation):
    class Meta:
        fleet_id = graphene.ID(required=True)

    ok = graphene.Boolean()
    srp_fleet = graphene.Field(SrpFleetMainType)

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, fleet_id):
        srpfleetmain = SrpFleetMain.objects.get(pk=fleet_id)
        srpfleetmain.fleet_srp_code = random_string(8)
        srpfleetmain.save()
        return cls(ok=True, srp_fleet=srpfleetmain)


class CompletedFleetMutation(graphene.Mutation):
    class Meta:
        fleet_id = graphene.ID(required=True)

    ok = graphene.Boolean()
    srp_fleet = graphene.Field(SrpFleetMainType)

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, fleet_id):
        srpfleetmain = SrpFleetMain.objects.get(pk=fleet_id)
        srpfleetmain.fleet_srp_status = "Completed"
        srpfleetmain.save()
        return cls(ok=True, srp_fleet=srpfleetmain)


class UncompletedFleetMutation(graphene.Mutation):
    class Meta:
        fleet_id = graphene.ID(required=True)

    ok = graphene.Boolean()
    srp_fleet = graphene.Field(SrpFleetMainType)

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, fleet_id):
        srpfleetmain = SrpFleetMain.objects.get(pk=fleet_id)
        srpfleetmain.fleet_srp_status = ""
        srpfleetmain.save()
        return cls(ok=True, srp_fleet=srpfleetmain)


class SrpFleetUserRequestFormMutation(DjangoFormMutation):
    class Meta:
        form_class = GQLSrpFleetUserRequestForm

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('srp.access_srp')
    def perform_mutate(cls, form: GQLSrpFleetUserRequestForm, info):
        request_killboard_link = form.cleaned_data['killboard_link']
        killmail_id = SRPManager.get_kill_id(killboard_link=request_killboard_link)

        # check if the killmail_id is already present
        if SrpUserRequest.objects.filter(killboard_link__icontains="/kill/" + killmail_id).exists():
            return cls(ok=False)

        character = info.context.user.profile.main_character
        srp_fleet_main = SrpFleetMain.objects.get(fleet_srp_code=form.cleaned_data['fleet_srp_code'])
        post_time = timezone.now()

        srp_request = SrpUserRequest()
        srp_request.killboard_link = request_killboard_link
        srp_request.additional_info = form.cleaned_data['additional_info']
        srp_request.character = character
        srp_request.srp_fleet_main = srp_fleet_main

        try:
            srp_kill_link = SRPManager.get_kill_id(srp_request.killboard_link)
            (ship_type_id, ship_value, victim_id) = SRPManager.get_kill_data(srp_kill_link)
        except ValueError:
            return cls(ok=False)

        if info.context.user.character_ownerships.filter(character__character_id=str(victim_id)).exists():
            item_type = esi.client.Universe.get_universe_types_type_id(type_id=ship_type_id).result()
            srp_request.srp_ship_name = item_type['name']
            srp_request.kb_total_loss = ship_value
            srp_request.post_time = post_time
            srp_request.save()
            return cls(ok=True)
        else:
            return cls(ok=False)


class SrpRequestRemoveMutation(graphene.Mutation):
    class Arguments:
        request_ids = graphene.List(graphene.ID, required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, request_ids):
        SrpUserRequest.objects.filter(id__in=request_ids).delete()
        return cls(ok=True)


class SrpRequestApproveMutation(graphene.Mutation):
    class Arguments:
        request_ids = graphene.List(graphene.ID, required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, request_ids):
        for srp_request_id in request_ids:
            if SrpUserRequest.objects.filter(id=srp_request_id).exists():
                srpuserrequest = SrpUserRequest.objects.get(id=srp_request_id)
                srpuserrequest.srp_status = "Approved"
                if srpuserrequest.srp_total_amount == 0:
                    srpuserrequest.srp_total_amount = srpuserrequest.kb_total_loss
                srpuserrequest.save()
                notify(
                    srpuserrequest.character.character_ownership.user,
                    'SRP Request Approved',
                    level='success',
                    message='Your SRP request for a {} lost during {} has been approved for {} ISK.'.format(
                        srpuserrequest.srp_ship_name, srpuserrequest.srp_fleet_main.fleet_name,
                        intcomma(srpuserrequest.srp_total_amount))
                )
        return cls(ok=True)


class SrpRequestRejectMutation(graphene.Mutation):
    class Arguments:
        request_ids = graphene.List(graphene.ID, required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, request_ids):
        for srp_request_id in request_ids:
            if SrpUserRequest.objects.filter(id=srp_request_id).exists():
                srpuserrequest = SrpUserRequest.objects.get(id=srp_request_id)
                srpuserrequest.srp_status = "Rejected"
                srpuserrequest.save()
                notify(
                    srpuserrequest.character.character_ownership.user,
                    'SRP Request Rejected',
                    level='danger',
                    message='Your SRP request for a {} lost during {} has been rejected.'.format(
                        srpuserrequest.srp_ship_name, srpuserrequest.srp_fleet_main.fleet_name)
                )
        return cls(ok=True)


class SrpUpdateAmountMutation(graphene.Mutation):
    class Arguments:
        fleet_srp_request_id = graphene.ID(required=True)
        srp_total_amount = graphene.Float(required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, fleet_srp_request_id, srp_total_amount):
        ok = False
        if SrpUserRequest.objects.filter(id=fleet_srp_request_id).exists():
            srp_request = SrpUserRequest.objects.get(id=fleet_srp_request_id)
            srp_request.srp_total_amount = srp_total_amount
            srp_request.save()
            ok = True
        return cls(ok=ok)


class SrpUpdateAARMutation(graphene.Mutation):
    class Arguments:
        fleet_id = graphene.ID(required=True)
        fleet_srp_aar_link = graphene.String(required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('auth.srp_management')
    def mutate(cls, root, info, fleet_id, fleet_srp_aar_link):
        srpfleetmain = SrpFleetMain.objects.get(id=fleet_id)
        srpfleetmain.fleet_srp_aar_link = fleet_srp_aar_link
        srpfleetmain.save()
        return cls(ok=True)


class Mutation:
    srp_add_fleet = AddFleetMutation.Field()
    srp_remove_fleet = RemoveFleetMutation.Field()
    srp_disable_fleet = DisableFleetMutation.Field()
    srp_enable_fleet = EnableFleetMutation.Field()
    srp_mark_completed = CompletedFleetMutation.Field()
    srp_mark_uncompleted = UncompletedFleetMutation.Field()
    srp_request = SrpFleetUserRequestFormMutation.Field()
    srp_remove_requests = SrpRequestRemoveMutation.Field()
    srp_approve_requests = SrpRequestApproveMutation.Field()
    srp_reject_requests = SrpRequestRejectMutation.Field()
    srp_update_amount = SrpUpdateAmountMutation.Field()
    srp_update_aar = SrpUpdateAARMutation.Field()



from functools import wraps
from graphql_jwt.exceptions import PermissionDenied
from graphql_jwt.decorators import context, user_passes_test

from esi.models import Token


def tokens_required(scopes, exc=PermissionDenied):
    def decorator(func):
        @wraps(func)
        @context(func)
        def _wrapped_func(context, *args, **kwargs):
            tokens = Token.objects\
                .filter(user__pk=context.user.pk)\
                .require_scopes(scopes)\
                .require_valid()

            if tokens.exists():
                return func(*args, **kwargs)
            else:
                raise exc('Required token missing')
        return _wrapped_func
    return decorator


def permissions_required(perm, raise_exception=False):
    """
    Decorator for views that checks whether a user has a particular permission
    enabled.
    If the raise_exception parameter is given the PermissionDenied exception
    is raised.

    This decorator is the graphql version of the allianceauth permission_required.
    """
    def check_perms(user):
        if isinstance(perm, str):
            perms = (perm,)
        else:
            perms = perm

        for perm_ in perms:
            perm_ = (perm_,)
            if user.has_perms(perm_):
                return True

        if raise_exception:
            raise PermissionDenied

        return False
    return user_passes_test(check_perms)



import graphene
from graphene_django import DjangoObjectType

from allianceauth.hrapplications.models import Application, ApplicationForm, ApplicationChoice, ApplicationQuestion, ApplicationResponse, ApplicationComment


class ApplicationStatus(graphene.Enum):
    PENDING = 1
    APPROVED = 2
    REJECTED = 3


class ApplicationType(DjangoObjectType):
    status = graphene.Field(ApplicationStatus, required=True)

    class Meta:
        model = Application
        fields = ('id', 'user', 'form', 'created', 'responses')

    def resolve_status(self, info):
        if self.approved is None:
            return ApplicationStatus.PENDING
        if self.approved:
            return ApplicationStatus.APPROVED
        return ApplicationStatus.REJECTED


class ApplicationAdminType(DjangoObjectType):
    status = graphene.Field(ApplicationStatus, required=True)

    class Meta:
        model = Application
        exclude = ('approved',)

    def resolve_status(self, info):
        if self.approved is None:
            return ApplicationStatus.PENDING
        if self.approved:
            return ApplicationStatus.APPROVED
        return ApplicationStatus.REJECTED


class ApplicationFormType(DjangoObjectType):
    class Meta:
        model = ApplicationForm
        fields = ('id', 'questions', 'corp', )


class ApplicationChoiceType(DjangoObjectType):
    class Meta:
        model = ApplicationChoice


class ApplicationQuestionType(DjangoObjectType):
    class Meta:
        model = ApplicationQuestion
        fields = ('id', 'title', 'help_text', 'multi_select', 'choices',)


class ApplicationResponseType(DjangoObjectType):
    class Meta:
        model = ApplicationResponse


class ApplicationCommentType(DjangoObjectType):
    class Meta:
        model = ApplicationComment



import graphene
from graphene_django import DjangoObjectType

from esi.models import Token, Scope

from allianceauth.eveonline.models import EveCharacter


class ScopeType(DjangoObjectType):
    friendly_name = graphene.String(required=True)

    class Meta:
        model = Scope
        fields = ('name', 'help_text',)


class TokenType(DjangoObjectType):
    character = graphene.Field('allianceauth_graphql.eveonline.types.EveCharacterType')

    class Meta:
        model = Token
        fields = ('id', 'user', 'scopes',)

    def resolve_character(self, info):
        return EveCharacter.objects.get(character_id=self.character_id)



from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models import Count, F, Q

import graphene
from graphql_jwt.decorators import login_required, permission_required

from .types import PermissionType, AppModelType


class Query:
    perms_search = graphene.List(
        PermissionType,
        app_label=graphene.String(),
        model=graphene.String(),
        search_string=graphene.String(),
        show_only_applied=graphene.Boolean(default_value=True)
    )

    perms_list_app_models = graphene.List(AppModelType)

    @login_required
    @permission_required('permissions_tool.audit_permissions')
    def resolve_perms_search(self, info, show_only_applied, app_label=None, model=None, search_string=None):
        perms = (
            Permission.objects.all()
            .annotate(num_users=Count('user', distinct=True))
            .annotate(num_groups=Count('group', distinct=True))
            .annotate(num_users_in_groups=Count('group__user', distinct=True))
            .annotate(num_states=Count('state', distinct=True))
            .annotate(num_users_in_states=Count('state__userprofile', distinct=True))
        )

        if show_only_applied:
            perms = perms.alias(num_entities=F('num_users') + F('num_groups') + F('num_states'))\
                .filter(num_entities__gt=0)

        if app_label:
            perms = perms.filter(content_type__app_label__icontains=app_label)

        if model:
            perms = perms.filter(content_type__model__icontains=model)

        if search_string:
            perms = perms.filter(
                Q(content_type__app_label__icontains=search_string) |
                Q(content_type__model__icontains=search_string) |
                Q(name__icontains=search_string) |
                Q(codename__icontains=search_string)
            )

        return perms

    @login_required
    @permission_required('permissions_tool.audit_permissions')
    def resolve_perms_list_app_models(self, info):
        return ContentType.objects.values('app_label', 'model')



import graphene
from graphene_django import DjangoObjectType

from allianceauth.timerboard.models import Timer, TimerType


TimerStructureChoices = graphene.Enum('TimerStructureChoices', [('POCO', 'POCO'),
                                                                ('I_HUB', 'I-HUB'),
                                                                ('TCU', 'TCU'),
                                                                ('POS_S', 'POS[S]'),
                                                                ('POS_M', 'POS[M]'),
                                                                ('POS_L', 'POS[L]'),
                                                                ('Astrahus', 'Astrahus'),
                                                                ('Fortizar', 'Fortizar'),
                                                                ('Keepstar', 'Keepstar'),
                                                                ('Raitaru', 'Raitaru'),
                                                                ('Azbel', 'Azbel'),
                                                                ('Sotiyo', 'Sotiyo'),
                                                                ('Athanor', 'Athanor'),
                                                                ('Tatara', 'Tatara'),
                                                                ('Pharolux_Cyno_Beacon', 'Pharolux Cyno Beacon'),
                                                                ('Tenebrex_Cyno_Jammer', 'Tenebrex Cyno Jammer'),
                                                                ('Ansiblex_Jump_Gate', 'Ansiblex Jump Gate'),
                                                                ('Moon_Mining_Cycle', 'Moon Mining Cycle'),
                                                                ('Other', 'Other')])
TimerObjectiveChoices = graphene.Enum('TimerObjectiveChoices', [('Friendly', 'Friendly'),
                                                                ('Hostile', 'Hostile'),
                                                                ('Neutral', 'Neutral')])
TimerTypeChoices = graphene.Enum.from_enum(TimerType)


class StructureTimerType(DjangoObjectType):
    class Meta:
        model = Timer
        exclude = ('eve_corp', 'eve_character',)



import graphene
from graphql_jwt.decorators import user_passes_test, login_required

from allianceauth.corputils.views import access_corpstats_test
from allianceauth.corputils.models import CorpStats, CorpMember

from .types import CorpStatsType, CorpMemberType


class Query:
    get_corpstats = graphene.List(CorpStatsType)
    get_corpstats_corp = graphene.Field(CorpStatsType, corp_id=graphene.Int(required=True))
    search_corpstats = graphene.List(CorpMemberType, search_string=graphene.String(required=True))

    @login_required
    @user_passes_test(access_corpstats_test)
    def resolve_get_corpstats(self, info):
        return CorpStats.objects.visible_to(info.context.user).order_by('corp__corporation_name').select_related('corp')

    @login_required
    @user_passes_test(access_corpstats_test)
    def resolve_get_corpstats_corp(self, info, corp_id):
        avaiable = CorpStats.objects.visible_to(info.context.user)
        stats = CorpStats.objects.get(corp__corporation_id=corp_id)
        if stats in avaiable:
            return stats

    @login_required
    @user_passes_test(access_corpstats_test)
    def resolve_search_corpstats(self, info, search_string):
        # original code, inefficient
        # results = []
        # has_similar = CorpStats.objects.filter(members__character_name__icontains=search_string).visible_to(info.context.user).distinct()
        # for corpstats in has_similar:
        #     similar = corpstats.members.filter(character_name__icontains=search_string)
        #     for s in similar:
        #         results.append((corpstats, s))
        # results = sorted(results, key=lambda x: x[1].character_name)

        # my code
        avaiable = CorpStats.objects.visible_to(info.context.user)
        return CorpMember.objects\
            .filter(corpstats__in=avaiable, character_name__icontains=search_string)\
            .order_by('character_name')



import graphene
from graphene_django import DjangoObjectType
from allianceauth.authentication.models import UserProfile, State, CharacterOwnership
from django.contrib.auth.models import Group, User


class StateType(DjangoObjectType):
    class Meta:
        model = State
        fields = ('name', )


class UserProfileType(DjangoObjectType):
    state = graphene.Field(StateType, required=True)

    class Meta:
        model = UserProfile


class UserType(DjangoObjectType):
    class Meta:
        model = User
        fields = ('id', 'username', 'profile', 'character_ownerships',)


class GroupStatusEnum(graphene.Enum):
    JOINED = 1
    PENDING = 2
    CAN_JOIN = 3
    CAN_APPLY = 4


class GroupType(DjangoObjectType):
    status = graphene.Field(GroupStatusEnum)
    num_members = graphene.Int()

    class Meta:
        model = Group
        fields = ('name', 'authgroup', 'id',)


class CharacterOwnershipType(DjangoObjectType):
    class Meta:
        model = CharacterOwnership
        fields = ('character', 'user',)


class LoginStatus(graphene.Enum):
    ERROR = 0
    LOGGED_IN = 1
    REGISTRATION = 2



import datetime

import graphene
from graphql_jwt.decorators import login_required, permission_required

from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.functions import ExtractMonth, ExtractYear
from django.db.models import Count

from allianceauth.fleetactivitytracking.models import Fatlink, Fat
from allianceauth.fleetactivitytracking.views import MemberStat, first_day_of_next_month, CorpStat
from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCorporationInfo, EveCharacter

from .types import FatlinkType, FatType, FatUserStatsType, FatCorpStatsType, FatPersonalStatsType, FatPersonalMonthlyStatsType


User = get_user_model()


class Query:
    recent_fat = graphene.List(FatType, num=graphene.Int())
    fatlinks = graphene.List(FatlinkType, num=graphene.Int())
    fat_corp_monthly_stats = graphene.List(FatUserStatsType, corp_id=graphene.Int(required=True), year=graphene.Int(required=True), month=graphene.Int(required=True))
    fat_general_monthly_stats = graphene.List(FatCorpStatsType, year=graphene.Int(required=True), month=graphene.Int(required=True))
    fat_personal_stats = graphene.List(FatPersonalStatsType)
    fat_personal_monthly_stats = graphene.Field(FatPersonalMonthlyStatsType, year=graphene.Int(required=True), month=graphene.Int(required=True), char_id=graphene.Int())

    @login_required
    def resolve_recent_fat(self, info, num=5):
        return Fat.objects.filter(user=info.context.user).order_by('-id')[:num]

    @login_required
    @permission_required('auth.fleetactivitytracking')
    def resolve_fatlinks(self, info, num=5):
        return Fatlink.objects.order_by('-id')[:num]

    @login_required
    @permission_required('auth.fleetactivitytracking_statistics')
    def resolve_fat_corp_monthly_stats(self, info, corp_id, year, month):
        start_of_month = datetime.datetime(year, month, 1)
        start_of_next_month = first_day_of_next_month(year, month)

        corp_members = CharacterOwnership.objects.filter(character__corporation_id=corp_id).order_by('user_id').values('user_id').distinct()

        fat_stats = {}

        for member in corp_members:
            try:
                fat_stats[member['user_id']] = MemberStat(User.objects.get(pk=member['user_id']), start_of_month, start_of_next_month)
            except ObjectDoesNotExist:
                continue

        stat_list = [{
            'user': x.mainchar.character_ownership.user,
            'num_chars': x.n_chars,
            'num_fats': x.n_fats,
            'average_fats': x.avg_fat
        } for x in fat_stats]
        stat_list.sort(key=lambda stat: stat['user'].profile.main_character.character_name)
        stat_list.sort(key=lambda stat: (stat['num_fats'], stat['avg_fat']), reverse=True)

        return stat_list

    @login_required
    @permission_required('auth.fleetactivitytracking_statistics')
    def resolve_fat_general_monthly_stats(self, info, year, month):
        start_of_month = datetime.datetime(year, month, 1)
        start_of_next_month = first_day_of_next_month(year, month)

        fat_stats = {}

        for corp in EveCorporationInfo.objects.all():
            fat_stats[corp.corporation_id] = CorpStat(corp.corporation_id, start_of_month, start_of_next_month)

        # get FAT stats for corps without models
        fats_in_span = Fat.objects.filter(fatlink__fatdatetime__gte=start_of_month).filter(
            fatlink__fatdatetime__lt=start_of_next_month).exclude(character__corporation_id__in=fat_stats)

        for fat in fats_in_span.exclude(character__corporation_id__in=fat_stats):
            if EveCorporationInfo.objects.filter(corporation_id=fat.character.corporation_id).exists():
                fat_stats[fat.character.corporation_id] = CorpStat(fat.character.corporation_id, start_of_month, start_of_next_month)

        # collect and sort stats
        stat_list = [{
            'corporation': x.corp,
            'num_fats': x.n_fats,
            'avg_fats': x.avg_fat,
        } for x in fat_stats]
        stat_list.sort(key=lambda stat: stat['corporation'].corporation_name)
        stat_list.sort(key=lambda stat: (stat['num_fats'], stat['avg_fat']), reverse=True)

        return stat_list

    @login_required
    def resolve_fat_personal_stats(self, info):
        # my optimized code
        return Fat.objects.filter(user=info.context.user)\
            .annotate(month=ExtractMonth('fatlink__fatdatetime'))\
            .annotate(year=ExtractYear('fatlink__fatdatetime'))\
            .values('month', 'year')\
            .annotate(num_fats=Count('*'))\
            .order_by('-year', '-month')

    @login_required
    def resolve_fat_personal_monthly_stats(self, info, year, month, char_id=None):
        if char_id and info.context.user.has_perm('auth.fleetactivitytracking_statistics'):
            user = EveCharacter.objects.get(character_id=char_id).character_ownership.user
        else:
            user = info.context.user

        # my optimized code
        personal_fats = Fat.objects.filter(user=user, fatlink__fatdatetime__year=year, fatlink__fatdatetime__month=month)\
            .values('shiptype')\
            .annotate(times_used=Count('*'))\
            .order_by('shiptype')

        created_fats = Fatlink.objects.filter(creator=user, fatdatetime__year=year, fatdatetime__month=month)

        return {
            'collected_links': personal_fats,
            'created_links': created_fats,
        }



import datetime

import graphene
from graphql_jwt.decorators import login_required, permission_required
from graphene_django.forms.mutation import DjangoFormMutation

from django.utils import timezone
from django.core.exceptions import ValidationError
from django.utils.crypto import get_random_string

from allianceauth.fleetactivitytracking.models import Fatlink, Fat
from allianceauth.fleetactivitytracking.views import SWAGGER_SPEC_PATH
from allianceauth.fleetactivitytracking.forms import FatlinkForm
from allianceauth.eveonline.models import EveCharacter
from allianceauth.eveonline.providers import provider
from esi.models import Token

from .types import FatlinkType
from ..decorators import tokens_required


class AddFatParticipation(graphene.Mutation):
    class Arguments:
        token_id = graphene.ID(required=True)
        fatlink_hash = graphene.String(required=True)

    ok = graphene.Boolean()
    error = graphene.String()

    @classmethod
    @login_required
    @tokens_required(scopes=['esi-location.read_location.v1', 'esi-location.read_ship_type.v1', 'esi-universe.read_structures.v1'])
    def mutate(cls, root, info, token_id, fatlink_hash):
        fatlink = Fatlink.objects.get(hash=fatlink_hash)
        token = Token.objects.get(pk=token_id)

        if (token.user == info.context.user) and (timezone.now() - fatlink.fatdatetime) < datetime.timedelta(seconds=(fatlink.duration * 60)):
            character = EveCharacter.objects.get_character_by_id(token.character_id)

            if character:
                c = token.get_esi_client(spec_file=SWAGGER_SPEC_PATH)
                location = c.Location.get_characters_character_id_location(character_id=token.character_id).result()
                ship = c.Location.get_characters_character_id_ship(character_id=token.character_id).result()
                location['solar_system_name'] = \
                    c.Universe.get_universe_systems_system_id(system_id=location['solar_system_id']).result()['name']
                if location['station_id']:
                    location['station_name'] = \
                        c.Universe.get_universe_stations_station_id(station_id=location['station_id']).result()['name']
                elif location['structure_id']:
                    location['station_name'] = \
                        c.Universe.get_universe_structures_structure_id(structure_id=location['structure_id']).result()[
                            'name']
                else:
                    location['station_name'] = "No Station"
                ship['ship_type_name'] = provider.get_itemtype(ship['ship_type_id']).name

                fat = Fat()
                fat.system = location['solar_system_name']
                fat.station = location['station_name']
                fat.shiptype = ship['ship_type_name']
                fat.fatlink = fatlink
                fat.character = character
                fat.user = info.context.user
                try:
                    fat.full_clean()
                    fat.save()
                    ok = True
                    error = None
                except ValidationError as e:
                    err_messages = []
                    for errorname, message in e.message_dict.items():
                        err_messages.append(message[0])
                    error = ' '.join(err_messages)
                    ok = False
            else:
                ok = False
                error = "Character doesn't exists"
        else:
            ok = False
            error = "FAT link has expired or user not valid"

        return cls(ok=ok, error=error)


class CreateFatlink(DjangoFormMutation):
    class Meta:
        form_class = FatlinkForm

    ok = graphene.Boolean()
    errors = graphene.List(graphene.String)
    fatlink = graphene.Field(FatlinkType)

    @classmethod
    @login_required
    @permission_required('auth.fleetactivitytracking')
    def perform_mutate(cls, form, info):
        fatlink = Fatlink()
        fatlink.fleet = form.cleaned_data["fleet"]
        fatlink.duration = form.cleaned_data["duration"]
        fatlink.fatdatetime = timezone.now()
        fatlink.creator = info.context.user
        fatlink.hash = get_random_string(length=15)
        try:
            fatlink.full_clean()
            fatlink.save()
            ok = True
            errors = []
        except ValidationError as e:
            form = FatlinkForm()
            errors = []
            for errorname, message in e.message_dict.items():
                errors.append(message[0].decode())
            ok = False

        return cls(ok=ok, errors=errors, fatlink=fatlink if ok else None)


class RemoveCharFatlink(graphene.Mutation):
    class Arguments:
        fat_hash = graphene.String(required=True)
        character_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    fatlink = graphene.Field(FatlinkType)

    @classmethod
    @login_required
    @permission_required('auth.fleetactivitytracking')
    def mutate(cls, root, info, fat_hash, character_id):
        fatlink = Fatlink.objects.get(hash=fat_hash)
        Fat.objects.filter(fatlink=fatlink, character__character_id=character_id).delete()
        return cls(ok=True, fatlink=fatlink)


class DeleteFatlink(graphene.Mutation):
    class Arguments:
        fat_hash = graphene.String(required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    @permission_required('auth.fleetactivitytracking')
    def mutate(cls, root, info, fat_hash):
        Fatlink.objects.get(hash=fat_hash).delete()
        return cls(ok=True)


class Mutation:
    fat_partecipate_to_fatlink = AddFatParticipation.Field()
    fat_create_fatlink = CreateFatlink.Field()
    fat_remove_char_fat = RemoveCharFatlink.Field()
    fat_delete_fatlink = DeleteFatlink.Field()



from django import forms
from allianceauth.srp.form import SrpFleetUserRequestForm


class GQLSrpFleetUserRequestForm(SrpFleetUserRequestForm):
    fleet_srp_code = forms.CharField(max_length=254, required=True)



import graphene
from graphene_django import DjangoObjectType

from django.contrib.auth import get_user_model
from django.db.models import Q

from allianceauth.corputils.models import CorpStats, CorpMember
from allianceauth.eveonline.models import EveCharacter


User = get_user_model()


class CorpMemberType(DjangoObjectType):
    character = graphene.Field('allianceauth_graphql.eveonline.types.EveCharacterType')

    class Meta:
        model = CorpMember
        fields = ('corpstats', 'character_id', 'character_name',)

    def resolve_character(self, info):
        try:
            return EveCharacter.objects.get(character_id=self.character_id)
        except EveCharacter.DoesNotExist:
            return None


class CorpStatsType(DjangoObjectType):
    registered = graphene.List('allianceauth_graphql.eveonline.types.EveCharacterType')
    unregistered = graphene.List('allianceauth_graphql.eveonline.types.EveCharacterType')
    mains = graphene.List('allianceauth_graphql.eveonline.types.EveCharacterType')

    class Meta:
        model = CorpStats
        fields = ('corp', 'last_update', 'members',)

    def resolve_registered(self, info):
        return EveCharacter.objects.filter(character_id__in=self.members.values('character_id'), character_ownership__isnull=False)\
            .select_related('character_ownership', 'character_ownership__user__profile__main_character')\
            .prefetch_related('character_ownership__user__character_ownerships')\
            .prefetch_related('character_ownership__user__character_ownerships__character')

    def resolve_unregistered(self, info):
        return EveCharacter.objects.filter(character_id__in=self.members.values('character_id'), character_ownership__isnull=True)

    def resolve_mains(self, info):
        main_ids = User.objects.values('profile__main_character__character_id')
        return EveCharacter.objects.filter(Q(character_id__in=self.members.values('character_id')) & Q(character_id__in=main_ids))



import graphene
from graphql_jwt.decorators import login_required, user_passes_test

from django.contrib.auth.models import Group
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied

from allianceauth.groupmanagement.managers import GroupManager
from allianceauth.groupmanagement.models import RequestLog, GroupRequest
from allianceauth.services.hooks import get_extension_logger
from allianceauth.notifications import notify

from .types import GroupRequestAddStatus, GroupRequestLeaveStatus


logger = get_extension_logger(__name__)


class AddGroupRequest(graphene.Mutation):
    class Arguments:
        group_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    status = graphene.Field(GroupRequestAddStatus)

    @classmethod
    @login_required
    def mutate(cls, root, info, group_id):
        user = info.context.user
        group = Group.objects.get(id=group_id)

        logger.debug(f"group_request_add called by user {user} for group id {group_id}")

        if not GroupManager.joinable_group(group, user.profile.state):
            logger.warning(f"User {user} attempted to join group id {group_id} but it is not a joinable group")
            status = GroupRequestAddStatus.CANNOT_JOIN
            ok = False
        elif group in user.groups.all():
            logger.warning(f"User {user} attempted to join group id {group_id} but they are already a member.")
            status = GroupRequestAddStatus.ALREADY_MEMBER
            ok = False
        elif not user.has_perm('groupmanagement.request_groups') and not group.authgroup.public:
            logger.warning(f"User {user} attempted to join group id {group_id} but it is not a public group")
            status = GroupRequestAddStatus.CANNOT_JOIN
            ok = False
        elif group.authgroup.open:
            logger.info(f"{user} joining {group} as is an open group")
            user.groups.add(group)
            request_info = user.username + ":" + group.name
            log = RequestLog(request_type=False, group=group, request_info=request_info, action=1, request_actor=user)
            log.save()

            status = GroupRequestAddStatus.JOINED
            ok = True
        elif GroupRequest.objects.filter(user=user, group=group).exists():
            logger.info(f"{user} attempted to join {group} but already has an open application")
            status = GroupRequestAddStatus.ALREADY_MEMBER
            ok = False
        else:
            grouprequest = GroupRequest()
            grouprequest.group = group
            grouprequest.user = user
            grouprequest.leave_request = False
            grouprequest.save()
            logger.info(f"Created group request for user {user} to group {group}")
            grouprequest.notify_leaders()

            status = GroupRequestAddStatus.APPLIED
            ok = True

        return cls(ok=ok, status=status)


class LeaveGroupRequest(graphene.Mutation):
    class Arguments:
        group_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    status = graphene.Field(GroupRequestLeaveStatus)

    @classmethod
    @login_required
    def mutate(cls, root, info, group_id):
        user = info.context.user
        logger.debug(f"group_request_leave called by user {user} for group id {group_id}")
        group = Group.objects.get(id=group_id)

        if not GroupManager.check_internal_group(group):
            logger.warning(f"User {user} attempted to leave group id {group_id} but it is not a joinable group")
            status = GroupRequestLeaveStatus.CANNOT_LEAVE
            ok = False
        elif group not in user.groups.all():
            logger.debug(f"User {user} attempted to leave group id {group_id} but they are not a member")
            status = GroupRequestLeaveStatus.NOT_MEMBER
            ok = False
        elif group.authgroup.open:
            logger.info(f"{user} leaving {group} as is an open group")
            request_info = user.username + ":" + group.name
            log = RequestLog(request_type=True, group=group, request_info=request_info, action=1, request_actor=user)
            log.save()
            user.groups.remove(group)

            status = GroupRequestLeaveStatus.LEFT
            ok = True
        elif GroupRequest.objects.filter(user=user, group=group):
            logger.info(f"{user} attempted to leave {group} but already has an pending leave request.")
            status = GroupRequestLeaveStatus.PENDING_LEAVE_REQUEST
            ok = False
        elif getattr(settings, 'GROUPMANAGEMENT_AUTO_LEAVE', False):
            logger.info(f"{user} leaving joinable group {group} due to auto_leave")
            request_info = user.username + ":" + group.name
            log = RequestLog(request_type=True, group=group, request_info=request_info, action=1, request_actor=user)
            log.save()
            user.groups.remove(group)
        else:
            grouprequest = GroupRequest()
            grouprequest.group = group
            grouprequest.user = user
            grouprequest.leave_request = True
            grouprequest.save()
            logger.info(f"Created group leave request for user {user} to group {group}")
            grouprequest.notify_leaders()

            status = GroupRequestLeaveStatus.CREATED_LEAVE_REQUEST
            ok = True

        return cls(ok=ok, status=status)


class GroupMembershipRemove(graphene.Mutation):
    class Arguments:
        group_id = graphene.Int(required=True)
        user_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    error = graphene.String()

    @classmethod
    @login_required
    @user_passes_test(GroupManager.can_manage_groups)
    def mutate(cls, root, info, group_id, user_id):
        ok = False
        error = None
        a_user = info.context.user
        logger.debug(f"group_membership_remove called by user {a_user} for group id {group_id} on user id {user_id}")
        try:
            group = Group.objects.get(pk=group_id)
            # Check its a joinable group i.e. not corp or internal
            # And the user has permission to manage it
            if not GroupManager.check_internal_group(group) or not GroupManager.can_manage_group(a_user, group):
                logger.warning(f"User {a_user} attempted to remove a user from group {group_id} but permission was denied")
                error = 'Permission denied'
            else:
                try:
                    user = group.user_set.get(id=user_id)
                    request_info = user.username + ":" + group.name
                    log = RequestLog(request_type=None, group=group, request_info=request_info, action=1, request_actor=a_user)
                    log.save()
                    # Remove group from user
                    user.groups.remove(group)
                    logger.info(f"User {a_user} removed user {user} from group {group}")
                    ok = True
                except ObjectDoesNotExist:
                    error = "User does not exist in that group"

        except ObjectDoesNotExist:
            error = "Group does not exist"

        return cls(ok=ok, error=error)


class GroupMembershipAcceptRequest(graphene.Mutation):
    class Arguments:
        group_request_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    error = graphene.String()

    @classmethod
    @login_required
    @user_passes_test(GroupManager.can_manage_groups)
    def mutate(cls, root, info, group_request_id):
        user = info.context.user
        ok = True
        error = None
        logger.debug(f"group_accept_request called by user {user} for grouprequest id {group_request_id}")
        try:
            group_request = GroupRequest.objects.get(id=group_request_id)
        except GroupRequest.DoesNotExist:
            error = "Group request doesn't exist"
            ok = False
        else:
            try:
                group, created = Group.objects.get_or_create(name=group_request.group.name)

                if not GroupManager.joinable_group(group_request.group, group_request.user.profile.state) or \
                        not GroupManager.can_manage_group(user, group_request.group):
                    error = "Permission denied"
                    ok = False
                else:
                    group_request.user.groups.add(group)
                    group_request.user.save()
                    log = RequestLog(request_type=group_request.leave_request, group=group, request_info=group_request.__str__(), action=1, request_actor=user)
                    log.save()
                    group_request.delete()
                    logger.info("User {} accepted group request from user {} to group {}".format(
                        user, group_request.user, group_request.group.name))
                    notify(group_request.user, "Group Application Accepted", level="success",
                           message="Your application to %s has been accepted." % group_request.group)
            except PermissionDenied as p:
                logger.warning(f"User {user} attempted to accept group join request {group_request_id} but permission was denied")
                error = "Permission denied"
                ok = False
            except:
                error = 'An unhandled error occurred while processing the application from %(mainchar)s to %(group)s.' % {"mainchar": group_request.main_char, "group": group_request.group}
                logger.exception("Unhandled exception occurred while user {} attempting to accept grouprequest id {}.".format(user, group_request_id))
                ok = False

        return cls(ok=ok, error=error)


class GroupMembershipRejectRequest(graphene.Mutation):
    class Arguments:
        group_request_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    error = graphene.String()

    @classmethod
    @login_required
    @user_passes_test(GroupManager.can_manage_groups)
    def mutate(cls, root, info, group_request_id):
        user = info.context.user
        ok = True
        error = None

        logger.debug(f"group_reject_request called by user {user} for group request id {group_request_id}")
        try:
            group_request = GroupRequest.objects.get(id=group_request_id)
        except GroupRequest.DoesNotExist:
            error = "Group request doesn't exist"
            ok = False
        else:
            try:
                if not GroupManager.can_manage_group(user, group_request.group):
                    error = "Permission denied"
                    ok = False
                else:
                    logger.info("User {} rejected group request from user {} to group {}".format(
                        user, group_request.user, group_request.group.name))
                    log = RequestLog(request_type=group_request.leave_request, group=group_request.group, request_info=group_request.__str__(), action=0, request_actor=user)
                    log.save()
                    group_request.delete()
                    notify(group_request.user, "Group Application Rejected", level="danger", message="Your application to %s has been rejected." % group_request.group)

            except PermissionDenied as p:
                logger.warning(f"User {user} attempted to reject group join request {group_request_id} but permission was denied")
                error = "Permission denied"
                ok = False
            except:
                error = 'An unhandled error occurred while processing the application from %(mainchar)s to %(group)s.' % {"mainchar": group_request.main_char, "group": group_request.group}
                logger.exception("Unhandled exception occurred while user {} attempting to accept grouprequest id {}.".format(user, group_request_id))
                ok = False

        return cls(ok=ok, error=error)


class GroupLeaveAcceptRequest(graphene.Mutation):
    class Arguments:
        group_request_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    error = graphene.String()

    @classmethod
    @login_required
    @user_passes_test(GroupManager.can_manage_groups)
    def mutate(cls, root, info, group_request_id):
        user = info.context.user
        ok = True
        error = None
        logger.debug(
            f"group_leave_accept_request called by user {user} for group request id {group_request_id}")
        try:
            group_request = GroupRequest.objects.get(id=group_request_id)
        except GroupRequest.DoesNotExist:
            error = "Group request doesn't exist"
            ok = False
        else:
            try:
                if not GroupManager.can_manage_group(user, group_request.group):
                    error = "Permission denied"
                    ok = False
                else:
                    group, created = Group.objects.get_or_create(name=group_request.group.name)
                    group_request.user.groups.remove(group)
                    group_request.user.save()
                    log = RequestLog(request_type=group_request.leave_request, group=group_request.group, request_info=group_request.__str__(), action=1, request_actor=user)
                    log.save()
                    group_request.delete()
                    logger.info("User {} accepted group leave request from user {} to group {}".format(
                        user, group_request.user, group_request.group.name))
                    notify(group_request.user, "Group Leave Request Accepted", level="success",
                           message="Your request to leave %s has been accepted." % group_request.group)
            except PermissionDenied as p:
                logger.warning(f"User {user} attempted to accept group leave request {group_request_id} but permission was denied")
                error = "Permission denied"
                ok = False
            except:
                error = 'An unhandled error occurred while processing the application from %(mainchar)s to %(group)s.' % {"mainchar": group_request.main_char, "group": group_request.group}
                logger.exception("Unhandled exception occurred while user {} attempting to accept grouprequest id {}.".format(user, group_request_id))
                ok = False

        return cls(ok=ok, error=error)


class GroupLeaveRejectRequest(graphene.Mutation):
    class Arguments:
        group_request_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    error = graphene.String()

    @classmethod
    @login_required
    @user_passes_test(GroupManager.can_manage_groups)
    def mutate(cls, root, info, group_request_id):
        user = info.context.user
        ok = True
        error = None

        logger.debug(
            f"group_leave_reject_request called by user {user} for group request id {group_request_id}")
        try:
            group_request = GroupRequest.objects.get(id=group_request_id)
        except GroupRequest.DoesNotExist:
            error = "Group request doesn't exist"
            ok = False
        else:
            try:
                if not GroupManager.can_manage_group(user, group_request.group):
                    error = "Permission denied"
                    ok = False
                else:
                    log = RequestLog(request_type=group_request.leave_request, group=group_request.group, request_info=group_request.__str__(), action=0, request_actor=user)
                    log.save()
                    group_request.delete()
                    logger.info("User {} rejected group leave request from user {} for group {}".format(
                        user, group_request.user, group_request.group.name))
                    notify(group_request.user, "Group Leave Request Rejected", level="danger", message="Your request to leave %s has been rejected." % group_request.group)
            except PermissionDenied as p:
                logger.warning(f"User {user} attempted to reject group leave request {group_request_id} but permission was denied")
                error = "Permission denied"
                ok = False
            except:
                error = 'An unhandled error occurred while processing the application from %(mainchar)s to %(group)s.' % {"mainchar": group_request.main_char, "group": group_request.group}
                logger.exception("Unhandled exception occurred while user {} attempting to accept grouprequest id {}.".format(user, group_request_id))
                ok = False

        return cls(ok=ok, error=error)


class Mutation:
    add_group_request = AddGroupRequest.Field()
    leave_group_request = LeaveGroupRequest.Field()
    group_membership_remove = GroupMembershipRemove.Field()
    group_membership_accept_request = GroupMembershipAcceptRequest.Field()
    group_membership_reject_request = GroupMembershipRejectRequest.Field()
    group_leave_accept_request = GroupLeaveAcceptRequest.Field()



import graphene
from graphql_jwt.decorators import login_required

from allianceauth.notifications.models import Notification

from .types import NotificationType


class MarkNotifReadMutation(graphene.Mutation):
    class Arguments:
        notif_id = graphene.ID(required=True)

    ok = graphene.Boolean()
    notification = graphene.Field(NotificationType)

    @classmethod
    @login_required
    def mutate(cls, root, info, notif_id):
        notif = Notification.objects.get(pk=notif_id)
        if notif.user == info.context.user:
            notif.mark_viewed()
            ok = True
        else:
            ok = False

        return cls(ok=ok, notification=notif if ok else None)


class DeleteNotificationMutation(graphene.Mutation):
    class Arguments:
        notif_id = graphene.ID(required=True)

    ok = graphene.Boolean()

    @classmethod
    @login_required
    def mutate(cls, root, info, notif_id):
        notif = Notification.objects.get(pk=notif_id)

        if notif.user == info.context.user:
            notif.delete()
            ok = True
        else:
            ok = False

        return cls(ok=ok)


class AllReadMutation(graphene.Mutation):

    ok = graphene.Boolean()

    @classmethod
    @login_required
    def mutate(cls, root, info):
        Notification.objects.filter(user=info.context.user).update(viewed=True)
        return cls(ok=True)


class DeleteAllReadMutation(graphene.Mutation):

    ok = graphene.Boolean()

    @classmethod
    @login_required
    def mutate(cls, root, info):
        Notification.objects.filter(user=info.context.user, viewed=True).delete()
        return cls(ok=True)


class Mutation:
    notif_mark_as_read = MarkNotifReadMutation.Field()
    notif_delete = DeleteNotificationMutation.Field()
    notif_mark_all_read = AllReadMutation.Field()



import graphene
from graphql_jwt.decorators import login_required, permission_required

from django.db.models import Q

from allianceauth.hrapplications.models import Application, ApplicationForm

from .types import ApplicationType, ApplicationFormType, ApplicationStatus, ApplicationAdminType


class Query:
    hr_corp_applications = graphene.List(ApplicationAdminType)
    hr_finished_corp_applications = graphene.List(ApplicationAdminType)
    hr_list_avaiable_forms = graphene.List(ApplicationFormType)
    hr_personal_applications = graphene.List(ApplicationType, status=ApplicationStatus())
    hr_search_application = graphene.List(ApplicationAdminType, search_string=graphene.String(required=True))

    @login_required
    def resolve_hr_corp_applications(self, info):
        user = info.context.user
        main_char = user.profile.main_character
        res = Application.objects.none()

        if user.is_superuser:
            res = Application.objects.filter(approved=None).order_by('-created')
        elif user.has_perm('auth.human_resources') and main_char and ApplicationForm.objects.filter(corp__corporation_id=main_char.corporation_id).exists():
            res = Application.objects.filter(form__corp__corporation_id=main_char.corporation_id, approved=None).order_by('-created')

        return res

    @login_required
    def resolve_hr_finished_corp_applications(self, info):
        user = info.context.user
        main_char = user.profile.main_character
        res = Application.objects.none()

        if user.is_superuser:
            res = Application.objects.exclude(approved=None).order_by('-created')
        elif user.has_perm('auth.human_resources') and main_char and ApplicationForm.objects.filter(corp__corporation_id=main_char.corporation_id).exists():
            res = Application.objects.filter(form__corp__corporation_id=main_char.corporation_id)\
                .exclude(approved=None).order_by('-created')

        return res

    @login_required
    def resolve_hr_list_avaiable_forms(self, info):
        return ApplicationForm.objects.exclude(applications__user=info.context.user)

    @login_required
    def resolve_hr_personal_applications(self, info, status=None):
        res = info.context.user.applications.all()
        if status is not None:
            if status == ApplicationStatus.PENDING:
                approved = None
            elif status == ApplicationStatus.APPROVED:
                approved = True
            else:
                approved = False
            res = res.filter(approved=approved)
        return res

    @login_required
    @permission_required('auth.human_resources')
    def resolve_hr_search_application(self, info, search_string: str):
        user = info.context.user
        searchstring = search_string.lower()

        app_list = Application.objects.all()
        if not user.is_superuser:
            try:
                app_list = app_list.filter(
                    form__corp__corporation_id=user.profile.main_character.corporation_id)
            except AttributeError:
                return None

        return app_list.filter(
            Q(user__profile__main_character__character_name__icontains=searchstring) |
            Q(user__profile__main_character__corporation_name__icontains=searchstring) |
            Q(user__profile__main_character__alliance_name__icontains=searchstring) |
            Q(user__character_ownerships__character__character_name__icontains=searchstring) |
            Q(user__character_ownerships__character__corporation_name__icontains=searchstring) |
            Q(user__character_ownerships__character__alliance_name__icontains=searchstring) |
            Q(user__username__icontains=searchstring)
        )
